/* COPYRIGHT (C) 2001 Asher Langton All Rights Reserved. 
 * Parrondo.java is a simulator demonstrating Parrondo's paradox
 * author - Asher Langton (jlangton@sandiego.edu)
 * version 2.0, 19 January 2001
 */

import java.io.*;

public class Parrondo
{  
      private static final int PLAYS = 101;
      private static final double ALT_PROB = .5;
      private static final double BASEPROB1 = .5, BASEPROB2 = .1, BASEPROB3 = .75; 

      private double probC1, probC2, probC3; // the probablities of the coins
      private double epsilon;
      private double[] aOnly, bOnly, abAlt, abRand; // the data arrays
      private int trials, modulus, period;

      public Parrondo(int t, double eps, int mod, int per) throws IllegalArgumentException
      {
	 if (t < 1 || eps < 0.0 || eps > .095 || mod <=1 || mod > 100
	     || ((per % 2) != 0) || per < 2 || per > 100)
            throw new IllegalArgumentException("Bad values!");
	 trials = t;
	 epsilon = eps;
	 probC1 = BASEPROB1 - epsilon;
	 probC2 = BASEPROB2 - epsilon;
	 probC3 = BASEPROB3 - epsilon;
	 modulus = mod;
	 period = per;
	 runA();
	 runB();
	 runABalt();
	 runABrand();
      }

      public void printToFile(String filename) throws IOException
      {
	 PrintWriter pw = new PrintWriter(new FileOutputStream(filename)); 
	 pw.println("Epsilon = " + epsilon);
	 pw.println("Period = " + period);
	 pw.println("Modulus = " + modulus);
	 pw.println("GAME A ONLY: " + trials + " trials.");
	 pw.println("Iteration\tScore");
	 for (int i = 0; i < PLAYS; i++)
	    pw.println(i + "\t" + aOnly[i]);
	 pw.println("GAME B ONLY: " + trials + " trials.");
	 pw.println("Iteration\tScore");
	 for (int i = 0; i < PLAYS; i++)
	    pw.println(i + "\t" + bOnly[i]);
	 pw.println("GAME A AND B, ALTERNATING 2x2: " + trials
			+ " trials.");
	 pw.println("Iteration\tScore");
	 for (int i = 0; i < PLAYS; i++)
	    pw.println(i + "\t" + abAlt[i]);
	 pw.println("GAME A AND B, ALTERNATING RANDOMLY: " 
			+ trials + " trials.");
	 pw.println("Iteration\tScore");
	 for (int i = 0; i < PLAYS; i++)
	    pw.println(i + "\t" + abRand[i]);
	 pw.close();
      
      }
 
      // Runs GameA consecutively.   
      private void runA()
      {
	 int score,i,j;
	 int[] temp = new int[PLAYS]; //stores sum before avg 
	 aOnly = new double[PLAYS]; //average scores
	 for (i = 1; i <= trials; i++)  // compute the sum
	 {
	    score = 0;
	    for (j = 1; j < PLAYS; j++)
	    {
	       score = GameA(score);
	       temp[j] += score;
	    }
	 }
	 for (j = 1; j < PLAYS; j++)  // compute the average
	    aOnly[j] = ((double)temp[j])/((double)trials);
      }

      // Runs GameB consecutively.
      private void runB()
      {
	 int score,i,j;
	 int[] temp = new int[PLAYS]; //stores sum before avg
	 bOnly = new double[PLAYS]; //average scores
	 for (i = 1; i <= trials; i++)  //compute the sum
	 {
	    score = 0;
	    for (j = 1; j < PLAYS; j++)
	    {
	       score = GameB(score);
	       temp[j] += score;
	    }
	 }
	 for (j = 1; j < PLAYS; j++)  // compute the average
	    bOnly[j] = ((double)temp[j])/((double)trials);
      }
 
      // Alternates GameA and GameB, 2x2.   
      private void runABalt()
      {
	 int i,j,score;
	 int[] temp = new int[PLAYS]; //temp storage for scores
	 abAlt = new double[PLAYS]; //average scores
	 int z; // determines which game is played
	 for (i = 1; i <= trials; i++)  //compute the sum
	 {
	    score = 0;
	    for (j = 1; j < PLAYS; j++)
	    {
	       z = j % period;
	       if (z < period/2)
		  score = GameA(score);
	       else
		  score = GameB(score);
	       temp[j] += score;
	    }
	 }
	 for (j = 1; j < PLAYS; j++)  // compute the average
	    abAlt[j] = ((double)temp[j])/((double)trials);
      }

   
      // Alternates GameA and GameB randomly.
      private void runABrand()
      {
	 int score,i,j;
	 int[] temp = new int[PLAYS]; //temp storage for scores
	 abRand = new double[PLAYS]; //average scores
	 for (i = 1; i <= trials; i++)  //compute the sum
	 {
	    score = 0;
	    for (j = 1; j < PLAYS; j++)
	    {
	       if (Math.random() >= ALT_PROB)
		  score = GameA(score);
	       else
		  score = GameB(score);
	       temp[j] += score;
	    }
	 }
	 for (j = 1; j < PLAYS; j++)  // compute the average
	    abRand[j] = ((double)temp[j])/((double)trials);       
      }

      /** 
       * GameA wins, and returns an incremented score, if the random int flip
       * is less than or equal to probC1.
       * @param score
       *   the score prior to the play
       * @return 
       *   the updated score 
       */
      private int GameA(int score)
      {
	 if (Math.random() <= probC1)
	    score++;
	 else
	    score--;
	 return score;
      }

      /** 
       * GameB wins, and returns an incremented score, if the random int flip
       * is less than or equal to probC2 when the score is a multiple of
       * modulus, or less than probC3 when th score is not a multiple of
       * modulus.
       * @param score
       *   the score prior to the play
       * @return 
       *   the updated score 
       */
      private int GameB(int score)
      {
	 if (score % modulus == 0)
	 {
	    if (Math.random() <= probC2)
	       score++;
	    else
	       score--;
	 }
	 else
	 {
	    if (Math.random() <= probC3)
	       score++;
	    else
	       score--;
	 }
	 return score;
      }

      // basic accessor method
      public double[] getaOnly()
      {
	 return aOnly;
      }

      // basic accessor method
      public double[] getbOnly()
      {
	 return bOnly;
      }

      // basic accessor method
      public double[] getabAlt()
      {
	 return abAlt;
      }

      // basic accessor method
      public double[] getabRand()
      {
	 return abRand;
      }

      /** 
       * The main method generates numeric results that are saved to a file; 
       * not needed for graphic interface
       */
      public static void main(String[] args)
      {
	 String file = Kb.getWord("File to write to: ");
	 int t = Kb.getInt("Trials: ", 1, 1000000);
	 double eps = Kb.getDouble("Epsilon (usually .005): ", 0.0, .095);
	 int mod = Kb.getInt("Modulus (usually 3): ", 2, 100);
	 int per = 2 * Kb.getInt("Games between alternation (usually 2): ", 1, 50);
	 Parrondo par = new Parrondo(t,eps,mod,per);
	 try
	 {
	    par.printToFile(file);
	 }
	 catch (IOException e)
	 {
	    System.out.println("Couldn't write to that file!");
	    System.exit(1);
	 }
      }

   
 
}

